using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Net.Cache;
using System.Net;
using System.IO;

namespace EveSkillNotify
{
    public class EveApi
    {
        public class characterStatus
        {
            public string CharID = "";
            public string CharName = "";
            public DateTime SkillFinishes = DateTime.MinValue;
            public TimeSpan SkillCompletionETA = TimeSpan.Zero;
            public bool IsTraining = false;
            public int SkillID = 0;
            public string SkillName = "";
            public int TrainingToLevel = 0;
        }

        private Settings SkillTypeIDs;
        public List<characterStatus> characters = new List<characterStatus>();
        private string userID = "";
        private string apiKey = "";

        public EveApi(Settings inSkillTypeIDs)
        {
            SkillTypeIDs = inSkillTypeIDs;
        }

        public characterStatus TrainingStatus()
        {
            characterStatus trainingAccount = null;
            foreach (characterStatus cs in characters)
            {
                if (cs.IsTraining == true)
                    trainingAccount = cs;
            }
            if (trainingAccount != null)
            {
                //Update skill eta
                DateTime startTime = DateTime.Now;
                DateTime endTime = trainingAccount.SkillFinishes;
                trainingAccount.SkillCompletionETA = endTime.Subtract(startTime);
            }
            return trainingAccount;
        }
        public bool LoadCharacters()
        {
            XmlDocument charData = PostData("/account/Characters.xml.aspx", "userID="+userID+"&apiKey="+apiKey);
            if (charData == null || CheckForError(charData))
                return false;
            
            XmlNodeList nl = charData.GetElementsByTagName("row");
            foreach (XmlNode xnode in nl)
            {
                if (xnode.Attributes.Count != 4)
                    continue;
                characterStatus cs = new characterStatus();
                cs.CharID = xnode.Attributes["characterID"].InnerText;
                cs.CharName = xnode.Attributes["name"].InnerText;
                characters.Add(cs);
            }
            if (characters.Count == 0)
            {
                errorMessage = "Server returned no characters";
                return false;
            }
            return true;
        }
        private DateTime cachedUntil = DateTime.MinValue;
        public DateTime CachedUntil
        {
            get { return cachedUntil; }
        }
        public bool LoadSkills()
        {
            foreach (characterStatus character in characters)
            {
                XmlDocument charData = PostData("/char/SkillInTraining.xml.aspx",
                    "userID=" + userID + "&apiKey=" + apiKey + "&characterID=" + character.CharID);
                if (charData == null || CheckForError(charData))
                    return false;
                XmlNode cache = charData.SelectSingleNode("/eveapi/cachedUntil");
                if (cache != null)
                {
                    bool result = DateTime.TryParseExact(
                                cache.InnerText,
                                "yyyy-MM-dd H:m:s", //Api date format
                                System.Globalization.CultureInfo.InvariantCulture,
                                System.Globalization.DateTimeStyles.AssumeUniversal,
                                out cachedUntil);
                    
                }
                XmlNodeList nl = charData.SelectNodes("/eveapi/result/*");
                foreach (XmlNode xnode in nl)
                {
                    bool castSucessful = false;
                    switch (xnode.Name)
                    {
                        case "skillInTraining":
                            if (xnode.InnerText == "0")
                                character.IsTraining = false;
                            else if (xnode.InnerText == "1")
                                character.IsTraining = true;
                            break;
                        case "trainingTypeID":
                            castSucessful = int.TryParse(xnode.InnerText, out character.SkillID);
                            if (!castSucessful || !SkillTypeIDs.Contains(xnode.InnerText))
                                character.SkillName = "Unknown Skill ("+xnode.InnerText+")";
                            else
                                character.SkillName = SkillTypeIDs.Get(xnode.InnerText);
                            break;
                        case "trainingToLevel":
                            castSucessful = int.TryParse(xnode.InnerText, out character.TrainingToLevel);
                            if (!castSucessful)
                                character.TrainingToLevel = 10;
                            break;
                        case "trainingEndTime":
                            castSucessful = DateTime.TryParseExact(
                                xnode.InnerText,
                                "yyyy-MM-dd H:m:s", //Api date format
                                System.Globalization.CultureInfo.InvariantCulture,
                                System.Globalization.DateTimeStyles.AssumeUniversal,
                                out character.SkillFinishes);
                            if (castSucessful)
                            {
                                DateTime startTime = DateTime.Now;
                                DateTime endTime = character.SkillFinishes;
                                character.SkillCompletionETA = endTime.Subtract(startTime);
                            }
                            break;
                    }
                }
                if (nl.Count == 0)
                {
                    errorMessage = "Server returned no skill training information";
                }
            }
            return true;
        }
        public bool SetKey(string n_UserID, string n_ApiKey)
        {
            userID = n_UserID;
            apiKey = n_ApiKey;
            return true;
        }
        public bool CheckForError(XmlDocument xmldoc)
        {
            XmlNodeList nl = xmldoc.GetElementsByTagName("error");
            if (nl.Count > 0)
            {
                XmlNode xnode = nl[0];
                errorMessage = "Api Error ("+xnode.Attributes["code"].InnerText+"): "+xnode.InnerText;
                return true;
            }
            return false;
        }
        private XmlDocument PostData(string address, string postinfo)
        {
            try
            {
                byte[] postdata = Encoding.ASCII.GetBytes(postinfo);
                HttpWebRequest wq = (HttpWebRequest)WebRequest.Create("http://api.eve-online.com"+address);
                wq.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.NoCacheNoStore);
                wq.Method = "POST";
                wq.ContentType = "application/x-www-form-urlencoded";
                wq.ContentLength = postdata.Length;
                Stream poststream = wq.GetRequestStream();
                poststream.Write(postdata, 0, postdata.Length);
                poststream.Close();
                HttpWebResponse wr = (HttpWebResponse)wq.GetResponse();
                StreamReader responsereader = new StreamReader(wr.GetResponseStream());
                string responsetext = responsereader.ReadToEnd();
                XmlDocument output = new XmlDocument();
                output.LoadXml(responsetext);
                return output;
            }
            catch (Exception e)
            {
                errorMessage = "Could not retrieve information: "+e.Message;
            }
            return null;
        }

        private string errorMessage = "";
        public string ErrorText
        {
            get
            {
                return errorMessage;
            }
        }
        
    }
}
